"""
A child class to simulate raw data of conventional Sim.
"""
__author__ = "Meizhu Liang @Imperial College London"

from napari_generic_simulator.baseSIMulator import Base_simulator
import numpy as np


class ConSim_simulator(Base_simulator):
    '''
    Implements sinusoidal SIM illumination with two beams, three angles and three phase steps.

    eta is the factor by which the illumination grid frequency exceeds the incoherent cutoff, eta = 1 for normal
    SIM, eta=sqrt(3) / 2 to maximise resolution without zeros in TF.
    For a normal SIM, maximum resolution extension = 1 + eta
    carrier is 2 * kmax * eta
    '''

    def __init__(self):
        self._phaseStep = 3
        self._angleStep = 3
        super().__init__()
        self._eta = 0.5 * self.n / self.NA

    """All polarisations are normalised to average intensity of 1, and with theta being  π/2 for the light sheet"""

    def _illCi(self):
        # illumination with circular polarisation in 3 angles
        _illCi_0 = 1
        _illCi_1 = 1
        _illCi_2 = 1
        return _illCi_0, _illCi_1, _illCi_2

    def _illAx(self):
        # illumination with axial polarisation in 3 angles
        _illAx_0 = 1 + 1 / 3 * np.cos(self.ph * (-2 * self.x) + self.p1)
        _illAx_1 = 1 + 1 / 3 * np.cos(self.ph * (self.x - np.sqrt(3) * self.y) + self.p1)
        _illAx_2 = 1 + 1 / 3 * np.cos(self.ph * (self.x + np.sqrt(3) * self.y) + self.p1)
        return _illAx_0, _illAx_1, _illAx_2

    def _illIp(self):
        # illumination with in-plane polarisation in 3 angles
        _illIp_0 = 1 - 1 / 3 * np.cos(self.ph * (-2 * self.x) + self.p1)
        _illIp_1 = 1 - 1 / 3 * np.cos(self.ph * (self.x - np.sqrt(3) * self.y) + self.p1)
        _illIp_2 = 1 - 1 / 3 * np.cos(self.ph * (self.x + np.sqrt(3) * self.y) + self.p1)
        return _illIp_0, _illIp_1, _illIp_2



from napari_generic_simulator import SIMulator

# make_napari_viewer is a pytest fixture that returns a napari viewer object
# capsys is a pytest fixture that captures stdout and stderr output streams
def test_simulator_widget(make_napari_viewer, capsys):

    viewer = make_napari_viewer()
    simulator_widget = SIMulator(viewer)


"""
Two child classes to simulate raw data of HexSim.
"""
__author__ = "Meizhu Liang @Imperial College London"

from napari_generic_simulator.baseSIMulator import Base_simulator
import numpy as np


class HexSim_simulator(Base_simulator):
    '''
    Implements hexagonal SIM illumination with three beams, seven phase steps, with beams being at 2π/3 angles.

    eta is the factor by which the illumination grid frequency exceeds the incoherent cutoff, eta = 1 for normal
    SIM, eta=sqrt(3) / 2 to maximise resolution without zeros in TF.
    For a normal SIM, maximum resolution extension = 1 + eta
    carrier is 2 * kmax * eta
    '''

    def __init__(self):
        self._phaseStep = 7
        self._angleStep = 1
        super().__init__()
        self._eta = self.n * np.sqrt(3.0) / 2 / self.NA

    """All polarisations are normalised to average intensity of 1, and with theta being  π/2 for the light sheet"""
    def _illCi(self):
        # Circular polarisation
        _illCi = 1 + 1 / 6 * (np.cos(self.ph * (-np.sqrt(3) * self.x - self.y) / 2 + self.p2) \
                      + np.cos(self.ph * (-np.sqrt(3) * self.x + self.y) / 2 + self.p1) \
                      + np.cos(self.ph * self.y + self.p1 - self.p2))
        return [_illCi]

    def _illAx(self):
        # Axial polarisation
        _illAx = 1 + 2 / 3 * (np.cos(self.ph * self.y + self.p1 - self.p2) \
                      + np.cos(self.ph * (self.y - np.sqrt(3) * self.x) / 2 + self.p1) \
                      + np.cos(self.ph * (-self.y - np.sqrt(3) * self.x) / 2 + self.p2))
        return [_illAx]

    def _illIp(self):
        # In-plane polarisation
        _illIp = 1 - 1 / 3 * (np.cos(self.ph * self.y + self.p1 - self.p2) \
                      + np.cos(self.ph * (self.y - np.sqrt(3) * self.x) / 2 + self.p1) \
                      + np.cos(self.ph * (self.y + np.sqrt(3) * self.x) / 2 + self.p2))
        return [_illIp]

class RightHexSim_simulator(Base_simulator):
    '''
    Implements hexagonal SIM illumination with three beams, seven phase steps, with beams being at right angles.
    '''

    def __init__(self):
        self._phaseStep = 7
        self._angleStep = 1
        super().__init__()
        self._eta = self.n * np.sqrt(3.0) / 2 / self.NA

    def _illCi(self):
        # Circular polarisation
        _illCi = 1 + 1 / 3 * (np.cos(self.ph * (-self.x + self.y) + self.p1) +
                                   np.cos(self.ph * (self.x + self.y) + self.p1 - self.p2))
        return [_illCi]

    def _illAx(self):
        # Axial polarisation with theta being π/2
        _illAx = 1 + 2 / 3 * (np.cos(self.ph * self.y + self.p1 - self.p2) +
                                   np.cos(self.ph * (self.x + self.y) / 2 + self.p1) +
                                   np.cos(self.ph * (-self.x + self.y) / 2 + self.p2))
        return [_illAx]

    def _illIp(self):
        # In-plane polarisation
        _illIp = 1 - 2 / 3 * np.cos(self.ph * (-self.x) + self.p2)
        return [_illIp]


if __name__ == '__main__':
    s = HexSim_simulator()
    s.acc = 3
    for msg in s.raw_image_stack():
        print(msg)
